@@ -16,9 +16,12 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
+
+#define _BSD_SOURCE
+
 #include "config.h"
 
 #include <errno.h>
@@ -45,6 +48,10 @@
 #endif
 #include <fcntl.h>
 
+/* will happen only on Win32 */
+#ifndef HAVE_LSTAT
+#define lstat stat
+#endif
 
 #ifdef HAVE_LIBEXIF
 #include <libexif/exif-data.h>
@@ -54,6 +61,7 @@
 #include <gphoto2/gphoto2-library.h>
 #include <gphoto2/gphoto2-port.h>
 #include <gphoto2/gphoto2-port-log.h>
+#include <gphoto2/gphoto2-port-portability.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -117,6 +125,7 @@ static const struct {
 	{"srw",  GP_MIME_RAW},
 	{"gf1",  GP_MIME_RAW},
 	{"srw",  GP_MIME_RAW},
+	{"nrw",  GP_MIME_RAW},
 	{"png",  GP_MIME_PNG},
 	{"wav",  GP_MIME_WAV},
 	{"3gp",  "video/3gpp"},
@@ -370,6 +379,7 @@ folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 				gp_context_error (context, _("Could not get information "
 							     "about '%s' (%s)."),
 						  buf, strerror(saved_errno));
+				gp_system_closedir (dir);
 				return GP_ERROR;
 			}
 			if (S_ISDIR (st.st_mode)) {
@@ -555,6 +565,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		return GP_ERROR_NO_MEMORY;
 	}
 	if (-1 == fstat(fd,&stbuf)) {
+		free (buf);
 		close (fd);
 		return GP_ERROR_IO_READ;
 	}
