@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 /*
  * This is working, but unfinished!
@@ -41,8 +41,12 @@
 #include <sys/select.h>
 #endif
 
-#include <sys/socket.h>
-#include <netinet/in.h>
+#ifdef WIN32
+# include <winsock.h>
+#else
+# include <sys/socket.h>
+# include <netinet/in.h>
+#endif
 
 #include <gphoto2/gphoto2-library.h>
 #include <gphoto2/gphoto2-port-log.h>
@@ -90,15 +94,18 @@
 #define ptpip_cmd_param4	30
 #define ptpip_cmd_param5	34
 
+#define PTP_EVENT_CHECK			0x0000	/* waits for */
+#define PTP_EVENT_CHECK_FAST		0x0001	/* checks */
 static uint16_t ptp_ptpip_check_event (PTPParams* params);
+static uint16_t ptp_ptpip_event (PTPParams* params, PTPContainer* event, int wait);
 
 /* send / receive functions */
 uint16_t
 ptp_ptpip_sendreq (PTPParams* params, PTPContainer* req)
 {
-	unsigned int 		ret;
-	uint32_t		len = 18+req->Nparam*4;
-	unsigned char 		*request = malloc(len);
+	int		ret;
+	int		len = 18+req->Nparam*4;
+	unsigned char 	*request = malloc(len);
 
 	ptp_ptpip_check_event (params);
 
@@ -194,23 +201,16 @@ ptp_ptpip_evt_read (PTPParams* params, PTPIPHeader *hdr, unsigned char** data) {
 
 static uint16_t
 ptp_ptpip_check_event (PTPParams* params) {
-	fd_set		infds;
-	struct timeval	timeout;
-	int ret;
-	unsigned char*	data = NULL;
-	PTPIPHeader	hdr;
+	PTPContainer	event;
+	uint16_t	ret;
 
-	FD_ZERO(&infds);
-	FD_SET(params->evtfd, &infds);
-	timeout.tv_sec = 0;
-	timeout.tv_usec = 1;
-	if (1 != select (params->evtfd+1, &infds, NULL, NULL, &timeout))
-		return PTP_RC_OK;
-	ret = ptp_ptpip_evt_read (params, &hdr, &data);
+	event.Code = 0;
+	ret = ptp_ptpip_event (params, &event, PTP_EVENT_CHECK_FAST);
 	if (ret != PTP_RC_OK)
 		return ret;
-	gp_log (GP_LOG_DEBUG,"ptpip/check_event", "hdr type %d, length %d", hdr.type, hdr.length);
-	return PTP_RC_OK;
+	if (event.Code == 0)
+		return ret;
+	return ptp_add_event (params, &event);
 }
 
 #define ptpip_startdata_transid		0
@@ -225,7 +225,8 @@ ptp_ptpip_senddata (PTPParams* params, PTPContainer* ptp,
 		uint64_t size, PTPDataHandler *handler
 ) {
 	unsigned char	request[0x14];
-	int		ret, curwrite, towrite;
+	unsigned int	curwrite, towrite;
+	int		ret;
 	unsigned char*	xdata;
 
 	htod32a(&request[ptpip_type],PTPIP_START_DATA_PACKET);
@@ -383,7 +384,7 @@ ptp_ptpip_getresp (PTPParams* params, PTPContainer* resp)
 
 	resp->Code		= dtoh16a(&data[ptpip_resp_code]);
 	resp->Transaction_ID	= dtoh32a(&data[ptpip_resp_transid]);
-	n = (dtoh32(hdr.length) - ptpip_resp_param1)/sizeof(uint32_t);
+	n = (dtoh32(hdr.length) - sizeof(hdr) - ptpip_resp_param1)/sizeof(uint32_t);
 	switch (n) {
 	case 5: resp->Param5 = dtoh32a(&data[ptpip_resp_param5]);
 	case 4: resp->Param4 = dtoh32a(&data[ptpip_resp_param4]);
@@ -407,7 +408,8 @@ ptp_ptpip_init_command_request (PTPParams* params)
 {
 	char		hostname[100];
 	unsigned char*	cmdrequest;
-	int 		len,i,ret;
+	unsigned int	i;
+	int 		len, ret;
 	unsigned char	guid[16];
 	
 	ptp_nikon_getptpipguid(guid);
@@ -425,8 +427,8 @@ ptp_ptpip_init_command_request (PTPParams* params)
 		cmdrequest[ptpip_initcmd_name+i*2] = hostname[i];
 		cmdrequest[ptpip_initcmd_name+i*2+1] = 0;
 	}
-	htod16a(&cmdrequest[ptpip_initcmd_name+(strlen(hostname)+1)*2],PTPIP_VERSION_MAJOR);
-	htod16a(&cmdrequest[ptpip_initcmd_name+(strlen(hostname)+1)*2+2],PTPIP_VERSION_MINOR);
+	htod16a(&cmdrequest[ptpip_initcmd_name+(strlen(hostname)+1)*2],PTPIP_VERSION_MINOR);
+	htod16a(&cmdrequest[ptpip_initcmd_name+(strlen(hostname)+1)*2+2],PTPIP_VERSION_MAJOR);
 
 	gp_log_data ( "ptpip/init_cmd", (char*)cmdrequest, len);
 	ret = write (params->cmdfd, cmdrequest, len);
@@ -461,6 +463,9 @@ ptp_ptpip_init_command_ack (PTPParams* params)
 		return ret;
 	if (hdr.type != dtoh32(PTPIP_INIT_COMMAND_ACK)) {
 		gp_log (GP_LOG_ERROR, "ptpip/init_cmd_ack", "bad type returned %d", htod32(hdr.type));
+		free (data);
+		if (hdr.type == PTPIP_INIT_FAIL) /* likely reason is permission denied */
+			return PTP_RC_AccessDenied;
 		return PTP_RC_GeneralError;
 	}
 	params->eventpipeid = dtoh32a(&data[ptpip_cmdack_idx]);
@@ -509,11 +514,11 @@ ptp_ptpip_init_event_ack (PTPParams* params)
 	ret = ptp_ptpip_evt_read (params, &hdr, &data);
 	if (ret != PTP_RC_OK)
 		return ret;
+	free (data);
 	if (hdr.type != dtoh32(PTPIP_INIT_EVENT_ACK)) {
 		gp_log (GP_LOG_ERROR, "ptpip", "bad type returned %d\n", htod32(hdr.type));
 		return PTP_RC_GeneralError;
 	}
-	free (data);
 	return PTP_RC_OK;
 }
 
@@ -521,14 +526,75 @@ ptp_ptpip_init_event_ack (PTPParams* params)
 /* Event handling functions */
 
 /* PTP Events wait for or check mode */
-#define PTP_EVENT_CHECK			0x0000	/* waits for */
-#define PTP_EVENT_CHECK_FAST		0x0001	/* checks */
 
-static inline uint16_t
+#define ptpip_event_code    0
+#define ptpip_event_transid	2
+#define ptpip_event_param1	6
+#define ptpip_event_param2	10
+#define ptpip_event_param3	14
+
+static uint16_t
 ptp_ptpip_event (PTPParams* params, PTPContainer* event, int wait)
 {
-	fprintf(stderr,"event()\n");
+#ifndef WIN32
+	fd_set		infds;
+	struct timeval	timeout;
+	int ret;
+	unsigned char*	data = NULL;
+	PTPIPHeader	hdr;
+	int n;
+
+	while (1) {
+		FD_ZERO(&infds);
+		FD_SET(params->evtfd, &infds);
+		timeout.tv_sec = 0;
+		if (wait == PTP_EVENT_CHECK_FAST)
+			timeout.tv_usec = 1;
+		else
+			timeout.tv_usec = 1000; /* 1/1000 second  .. perhaps wait longer? */
+
+		ret = select (params->evtfd+1, &infds, NULL, NULL, &timeout);
+		if (1 != ret) {
+			if (-1 == ret) {
+				gp_log (GP_LOG_DEBUG,"ptpip/event", "select returned error, errno is %d", errno);
+				return PTP_ERROR_IO;
+			}
+			return PTP_ERROR_TIMEOUT;
+		}
+
+		ret = ptp_ptpip_evt_read (params, &hdr, &data);
+		if (ret != PTP_RC_OK)
+			return ret;
+		gp_log (GP_LOG_DEBUG,"ptpip/event", "hdr type %d, length %d", hdr.type, hdr.length);
+
+		if (dtoh32(hdr.type) == PTPIP_EVENT) {
+			break;
+		}
+
+		/* TODO: Handle cancel transaction and ping/pong
+		 * If not PTPIP_EVENT, process it and wait for next PTPIP_EVENT
+		 */
+		gp_log (GP_LOG_ERROR, "ptpip/event", "unknown/unhandled event type %d", dtoh32(hdr.type));
+	}
+
+	event->Code		= dtoh16a(&data[ptpip_event_code]);
+	event->Transaction_ID	= dtoh32a(&data[ptpip_event_transid]);
+	n = (dtoh32(hdr.length) - sizeof(hdr) - ptpip_event_param1)/sizeof(uint32_t);
+	switch (n) {
+	case 3: event->Param3 = dtoh32a(&data[ptpip_event_param3]);
+	case 2: event->Param2 = dtoh32a(&data[ptpip_event_param2]);
+	case 1: event->Param1 = dtoh32a(&data[ptpip_event_param1]);
+	case 0: break;
+	default:
+		gp_log( GP_LOG_ERROR, "ptpip/event", "response got %d parameters?", n);
+		break;
+	}
+	free (data);
 	return PTP_RC_OK;
+#else
+	gp_log( GP_LOG_ERROR, "ptpip/event", "not supported currently on Windows");
+	return PTP_RC_OK;
+#endif
 }
 
 uint16_t
@@ -612,6 +678,8 @@ ptp_ptpip_connect (PTPParams* params, const char *address) {
 	gp_log (GP_LOG_DEBUG,"ptpip/connect", "connecting to %s.", address);
 	if (NULL == strchr (address,':'))
 		return GP_ERROR_BAD_PARAMETERS;
+
+#ifdef HAVE_INET_ATON
 	addr = strdup (address);
 	if (!addr)
 		return GP_ERROR_NO_MEMORY;
@@ -656,18 +724,18 @@ ptp_ptpip_connect (PTPParams* params, const char *address) {
 		close (params->evtfd);
 		return GP_ERROR_IO;
 	}
-	ret = ptp_ptpip_init_command_request (params);
-	if (ret != PTP_RC_OK)
-		return translate_ptp_result (ret);
-	ret = ptp_ptpip_init_command_ack (params);
-	if (ret != PTP_RC_OK)
-		return translate_ptp_result (ret);
 	if (-1 == connect (params->evtfd, (struct sockaddr*)&saddr, sizeof(struct sockaddr_in))) {
 		perror ("connect evt");
 		close (params->cmdfd);
 		close (params->evtfd);
 		return GP_ERROR_IO;
 	}
+	ret = ptp_ptpip_init_command_request (params);
+	if (ret != PTP_RC_OK)
+		return translate_ptp_result (ret);
+	ret = ptp_ptpip_init_command_ack (params);
+	if (ret != PTP_RC_OK)
+		return translate_ptp_result (ret);
 	ret = ptp_ptpip_init_event_request (params);
 	if (ret != PTP_RC_OK)
 		return translate_ptp_result (ret);
@@ -676,4 +744,8 @@ ptp_ptpip_connect (PTPParams* params, const char *address) {
 		return translate_ptp_result (ret);
 	gp_log (GP_LOG_DEBUG, "ptpip/connect", "ptpip connected!");
 	return GP_OK;
+#else
+	gp_log (GP_LOG_ERROR,"ptpip/connect", "Windows currently not supported, neeeds a winsock port.");
+	return GP_ERROR_NOT_SUPPORTED;
+#endif
 }
